@@ -1,4 +1,4 @@
-// Copyright 2018-2021 Parity Technologies (UK) Ltd.
+// Copyright 2018-2022 Parity Technologies (UK) Ltd.
 // This file is part of cargo-contract.
 //
 // cargo-contract is free software: you can redistribute it and/or modify
@@ -34,7 +34,6 @@ use std::{
     process::Command,
     str,
 };
-use structopt::StructOpt;
 
 /// This is the maximum number of pages available for a contract to allocate.
 const MAX_MEMORY_PAGES: u32 = 16;
@@ -54,14 +53,14 @@ pub(crate) struct ExecuteArgs {
     output_type: OutputType,
 }
 
-/// Executes build of the smart-contract which produces a wasm binary that is ready for deploying.
+/// Executes build of the smart contract which produces a Wasm binary that is ready for deploying.
 ///
 /// It does so by invoking `cargo build` and then post processing the final binary.
-#[derive(Debug, StructOpt)]
-#[structopt(name = "build")]
+#[derive(Debug, clap::Args)]
+#[clap(name = "build")]
 pub struct BuildCommand {
-    /// Path to the Cargo.toml of the contract to build
-    #[structopt(long, parse(from_os_str))]
+    /// Path to the `Cargo.toml` of the contract to build
+    #[clap(long, parse(from_os_str))]
     manifest_path: Option<PathBuf>,
     /// By default the contract is compiled with debug functionality
     /// included. This enables the contract to output debug messages,
@@ -69,10 +68,10 @@ pub struct BuildCommand {
     ///
     /// A production contract should always be build in `release` mode!
     /// Then no debug functionality is compiled into the contract.
-    #[structopt(long = "--release")]
+    #[clap(long = "--release")]
     build_release: bool,
     /// Build offline
-    #[structopt(long = "--offline")]
+    #[clap(long = "--offline")]
     build_offline: bool,
     /// Which build artifacts to generate.
     ///
@@ -80,18 +79,18 @@ pub struct BuildCommand {
     ///
     /// - `code-only`: Only the Wasm is created, generation of metadata and a bundled
     ///   `<name>.contract` file is skipped.
-    #[structopt(
+    #[clap(
         long = "generate",
         default_value = "all",
         value_name = "all | code-only",
         verbatim_doc_comment
     )]
     build_artifact: BuildArtifacts,
-    #[structopt(flatten)]
+    #[clap(flatten)]
     verbosity: VerbosityFlags,
-    #[structopt(flatten)]
+    #[clap(flatten)]
     unstable_options: UnstableOptions,
-    /// Number of optimization passes, passed as an argument to wasm-opt.
+    /// Number of optimization passes, passed as an argument to `wasm-opt`.
     ///
     /// - `0`: execute no optimization passes
     ///
@@ -113,16 +112,16 @@ pub struct BuildCommand {
     /// - It is possible to define the number of optimization passes in the
     ///   `[package.metadata.contract]` of your `Cargo.toml` as e.g. `optimization-passes = "3"`.
     ///   The CLI argument always takes precedence over the profile value.
-    #[structopt(long)]
+    #[clap(long)]
     optimization_passes: Option<OptimizationPasses>,
     /// Do not remove symbols (Wasm name section) when optimizing.
     ///
     /// This is useful if one wants to analyze or debug the optimized binary.
-    #[structopt(long)]
+    #[clap(long)]
     keep_debug_symbols: bool,
 
     /// Export the build output in JSON format.
-    #[structopt(long, conflicts_with = "verbose")]
+    #[clap(long, conflicts_with = "verbose")]
     output_json: bool,
 }
 
@@ -184,15 +183,15 @@ impl BuildCommand {
     }
 }
 
-#[derive(Debug, StructOpt)]
-#[structopt(name = "check")]
+#[derive(Debug, clap::Args)]
+#[clap(name = "check")]
 pub struct CheckCommand {
-    /// Path to the Cargo.toml of the contract to build
-    #[structopt(long, parse(from_os_str))]
+    /// Path to the `Cargo.toml` of the contract to build
+    #[clap(long, parse(from_os_str))]
     manifest_path: Option<PathBuf>,
-    #[structopt(flatten)]
+    #[clap(flatten)]
     verbosity: VerbosityFlags,
-    #[structopt(flatten)]
+    #[clap(flatten)]
     unstable_options: UnstableOptions,
 }
 
@@ -226,14 +225,16 @@ impl CheckCommand {
 /// to build the standard library with [`panic_immediate_abort`](https://github.com/johnthagen/min-sized-rust#remove-panic-string-formatting-with-panic_immediate_abort)
 /// which reduces the size of the Wasm binary by not including panic strings and formatting code.
 ///
-/// # Cargo.toml optimizations
+/// # `Cargo.toml` optimizations
 ///
-/// The original Cargo.toml will be amended to remove the `rlib` crate type in order to minimize
+/// The original `Cargo.toml` will be amended to remove the `rlib` crate type in order to minimize
 /// the final Wasm binary size.
 ///
 /// Preferred default `[profile.release]` settings will be added if they are missing, existing
 /// user-defined settings will be preserved.
 ///
+/// The `[workspace]` will be added if it is missing to ignore `workspace` from parent `Cargo.toml`.
+///
 /// To disable this and use the original `Cargo.toml` as is then pass the `-Z original_manifest` flag.
 fn exec_cargo_for_wasm_target(
     crate_metadata: &CrateMetadata,
@@ -245,13 +246,6 @@ fn exec_cargo_for_wasm_target(
 ) -> Result<()> {
     util::assert_channel()?;
 
-    // set linker args via RUSTFLAGS.
-    // Currently will override user defined RUSTFLAGS from .cargo/config. See https://github.com/paritytech/cargo-contract/issues/98.
-    std::env::set_var(
-        "RUSTFLAGS",
-        "-C link-arg=-zstack-size=65536 -C link-arg=--import-memory -Clinker-plugin-lto",
-    );
-
     let cargo_build = |manifest_path: &ManifestPath| {
         let target_dir = &crate_metadata.target_directory;
         let target_dir = format!("--target-dir={}", target_dir.to_string_lossy());
@@ -270,7 +264,11 @@ fn exec_cargo_for_wasm_target(
         } else {
             args.push("-Zbuild-std-features=panic_immediate_abort");
         }
-        util::invoke_cargo(command, &args, manifest_path.directory(), verbosity)?;
+        let env = vec![(
+            "RUSTFLAGS",
+            Some("-C link-arg=-zstack-size=65536 -C link-arg=--import-memory -Clinker-plugin-lto"),
+        )];
+        util::invoke_cargo(command, &args, manifest_path.directory(), verbosity, env)?;
 
         Ok(())
     };
@@ -289,19 +287,129 @@ fn exec_cargo_for_wasm_target(
             .with_root_package_manifest(|manifest| {
                 manifest
                     .with_removed_crate_type("rlib")?
-                    .with_profile_release_defaults(Profile::default_contract_release())?;
+                    .with_profile_release_defaults(Profile::default_contract_release())?
+                    .with_workspace()?;
                 Ok(())
             })?
             .using_temp(cargo_build)?;
     }
 
-    // clear RUSTFLAGS
-    std::env::remove_var("RUSTFLAGS");
+    Ok(())
+}
+
+/// Executes `cargo dylint` with the ink! linting driver that is built during
+/// the `build.rs`.
+///
+/// We create a temporary folder, extract the linting driver there and run
+/// `cargo dylint` with it.
+fn exec_cargo_dylint(crate_metadata: &CrateMetadata, verbosity: Verbosity) -> Result<()> {
+    check_dylint_requirements(crate_metadata.manifest_path.directory())?;
+
+    let tmp_dir = tempfile::Builder::new()
+        .prefix("cargo-contract-dylint_")
+        .tempdir()?;
+    log::debug!("Using temp workspace at '{}'", tmp_dir.path().display());
+
+    let driver = include_bytes!(concat!(env!("OUT_DIR"), "/ink-dylint-driver.zip"));
+    crate::util::unzip(driver, tmp_dir.path().to_path_buf(), None)?;
+
+    let manifest_path = crate_metadata.manifest_path.cargo_arg()?;
+    let args = vec!["--lib", "ink_linting", &manifest_path];
+    let tmp_dir_path = tmp_dir.path().as_os_str().to_string_lossy();
+    let env = vec![
+        ("DYLINT_LIBRARY_PATH", Some(tmp_dir_path.as_ref())),
+        // For tests we need to set the `DYLINT_DRIVER_PATH` to a tmp folder,
+        // otherwise tests running in parallel will try to write to the same
+        // file at the same time which will result in a `Text file busy` error.
+        #[cfg(test)]
+        ("DYLINT_DRIVER_PATH", Some(tmp_dir_path.as_ref())),
+        // We need to remove the `CARGO_TARGET_DIR` environment variable in
+        // case `cargo dylint` is invoked.
+        //
+        // This is because the ink! dylint driver crate found in `dylint` uses a
+        // fixed Rust toolchain via the `ink_linting/rust-toolchain` file. By
+        // removing this env variable we avoid issues with different Rust toolchains
+        // interfering with each other.
+        ("CARGO_TARGET_DIR", None),
+        // There are generally problems with having a custom `rustc` wrapper, while
+        // executing `dylint` (which has a custom linker). Especially for `sccache`
+        // there is this bug: https://github.com/mozilla/sccache/issues/1000.
+        // Until we have a justification for leaving the wrapper we should unset it.
+        ("RUSTC_WRAPPER", None),
+    ];
+    let working_dir = crate_metadata
+        .manifest_path
+        .directory()
+        .unwrap_or_else(|| Path::new("."))
+        .canonicalize()?;
+
+    let verbosity = if verbosity == Verbosity::Verbose {
+        // `dylint` is verbose by default, it doesn't have a `--verbose` argument,
+        Verbosity::Default
+    } else {
+        verbosity
+    };
+    util::invoke_cargo("dylint", &args, Some(working_dir), verbosity, env)?;
 
     Ok(())
 }
 
-/// Ensures the wasm memory import of a given module has the maximum number of pages.
+/// Checks if all requirements for `dylint` are installed.
+///
+/// We require only an installed version of `cargo-dylint` here and don't
+/// check for an installed version of `dylint-link`. This is because
+/// `dylint-link` is only required for the `dylint` driver build process
+/// in `build.rs`.
+///
+/// This function takes a `_working_dir` which is only used for unit tests.
+fn check_dylint_requirements(_working_dir: Option<&Path>) -> Result<()> {
+    let execute_cmd = |cmd: &mut Command| {
+        // when testing this function we set the `PATH` to the `working_dir`
+        // so that we can have mocked binaries in there which are executed
+        // instead of the real ones.
+        #[cfg(test)]
+        {
+            let default_dir = PathBuf::from(".");
+            let working_dir = _working_dir.unwrap_or(default_dir.as_path());
+            let path_env = std::env::var("PATH").unwrap();
+            let path_env = format!("{}:{}", working_dir.to_string_lossy(), path_env);
+            cmd.env("PATH", path_env);
+        }
+
+        cmd.stdout(std::process::Stdio::null())
+            .stderr(std::process::Stdio::null())
+            .spawn()
+            .map_err(|err| {
+                log::debug!("Error spawning `{:?}`", cmd);
+                err
+            })?
+            .wait()
+            .map(|res| res.success())
+            .map_err(|err| {
+                log::debug!("Error waiting for `{:?}`: {:?}", cmd, err);
+                err
+            })
+    };
+
+    // when testing this function we should never fall back to a `cargo` specified
+    // in the env variable, as this would mess with the mocked binaries.
+    #[cfg(not(test))]
+    let cargo = std::env::var("CARGO").unwrap_or_else(|_| "cargo".to_string());
+    #[cfg(test)]
+    let cargo = "cargo";
+
+    if !execute_cmd(Command::new(cargo).arg("dylint").arg("--version"))? {
+        anyhow::bail!("cargo-dylint was not found!\n\
+            Make sure it is installed and the binary is in your PATH environment.\n\n\
+            You can install it by executing `cargo install cargo-dylint`."
+            .to_string()
+            .bright_yellow());
+    }
+
+    Ok(())
+}
+
+/// Ensures the Wasm memory import of a given module has the maximum number of pages.
 ///
 /// Iterates over the import section, finds the memory import entry if any and adjusts the maximum
 /// limit.
@@ -360,7 +468,7 @@ fn strip_exports(module: &mut Module) {
     }
 }
 
-/// Load and parse a wasm file from disk.
+/// Load and parse a Wasm file from disk.
 fn load_module<P: AsRef<Path>>(path: P) -> Result<Module> {
     let path = path.as_ref();
     parity_wasm::deserialize_file(path).context(format!(
@@ -369,9 +477,9 @@ fn load_module<P: AsRef<Path>>(path: P) -> Result<Module> {
     ))
 }
 
-/// Performs required post-processing steps on the wasm artifact.
+/// Performs required post-processing steps on the Wasm artifact.
 fn post_process_wasm(crate_metadata: &CrateMetadata) -> Result<()> {
-    // Deserialize wasm module from a file.
+    // Deserialize Wasm module from a file.
     let mut module =
         load_module(&crate_metadata.original_wasm).context("Loading of original wasm failed")?;
 
@@ -390,9 +498,9 @@ fn post_process_wasm(crate_metadata: &CrateMetadata) -> Result<()> {
     Ok(())
 }
 
-/// Attempts to perform optional wasm optimization using `binaryen`.
+/// Attempts to perform optional Wasm optimization using `binaryen`.
 ///
-/// The intention is to reduce the size of bloated wasm binaries as a result of missing
+/// The intention is to reduce the size of bloated Wasm binaries as a result of missing
 /// optimizations (or bugs?) between Rust and Wasm.
 fn optimize_wasm(
     crate_metadata: &CrateMetadata,
@@ -436,7 +544,7 @@ fn optimize_wasm(
 /// The supplied `optimization_level` denotes the number of optimization passes,
 /// resulting in potentially a lot of time spent optimizing.
 ///
-/// If successful, the optimized wasm is written to `dest_optimized`.
+/// If successful, the optimized Wasm is written to `dest_optimized`.
 fn do_optimization(
     dest_wasm: &OsStr,
     dest_optimized: &OsStr,
@@ -447,7 +555,7 @@ fn do_optimization(
     let which = which::which("wasm-opt");
     if which.is_err() {
         anyhow::bail!(
-            "wasm-opt not found! Make sure the binary is in your PATH environment.\n\
+            "wasm-opt not found! Make sure the binary is in your PATH environment.\n\n\
             We use this tool to optimize the size of your contract's Wasm binary.\n\n\
             wasm-opt is part of the binaryen package. You can find detailed\n\
             installation instructions on https://github.com/WebAssembly/binaryen#tools.\n\n\
@@ -462,7 +570,7 @@ fn do_optimization(
     }
     let wasm_opt_path = which
         .as_ref()
-        .expect("we just checked if which returned an err; qed")
+        .expect("we just checked if `which` returned an err; qed")
         .as_path();
     log::info!("Path to wasm-opt executable: {}", wasm_opt_path.display());
 
@@ -507,23 +615,32 @@ fn do_optimization(
     Ok(())
 }
 
-/// Checks if the wasm-opt binary under `wasm_opt_path` returns a version
+/// Checks if the `wasm-opt` binary under `wasm_opt_path` returns a version
 /// compatible with `cargo-contract`.
 ///
 /// Currently this must be a version >= 99.
 fn check_wasm_opt_version_compatibility(wasm_opt_path: &Path) -> Result<()> {
-    let cmd = Command::new(wasm_opt_path)
-        .arg("--version")
-        .output()
-        .map_err(|err| {
-            anyhow::anyhow!(
-                "Executing `{:?} --version` failed with {:?}",
-                wasm_opt_path.display(),
-                err
-            )
-        })?;
-    if !cmd.status.success() {
-        let err = str::from_utf8(&cmd.stderr)
+    let mut cmd_res = Command::new(wasm_opt_path).arg("--version").output();
+
+    // The following condition is a workaround for a spurious CI failure:
+    // ```
+    // Executing `"/tmp/cargo-contract.test.GGnC0p/wasm-opt-mocked" --version` failed with
+    // Os { code: 26, kind: ExecutableFileBusy, message: "Text file busy" }
+    // ```
+    if cmd_res.is_err() && format!("{:?}", cmd_res).contains("ExecutableFileBusy") {
+        std::thread::sleep(std::time::Duration::from_secs(1));
+        cmd_res = Command::new(wasm_opt_path).arg("--version").output();
+    }
+
+    let res = cmd_res.map_err(|err| {
+        anyhow::anyhow!(
+            "Executing `{:?} --version` failed with {:?}",
+            wasm_opt_path.display(),
+            err
+        )
+    })?;
+    if !res.status.success() {
+        let err = str::from_utf8(&res.stderr)
             .expect("Cannot convert stderr output of wasm-opt to string")
             .trim();
         anyhow::bail!(
@@ -542,7 +659,7 @@ fn check_wasm_opt_version_compatibility(wasm_opt_path: &Path) -> Result<()> {
         way forward is to download a recent binary release directly:\n\n\
         https://github.com/WebAssembly/binaryen/releases\n\n\
         Make sure that the `wasm-opt` file from that release is in your `PATH`.";
-    let version_stdout = str::from_utf8(&cmd.stdout)
+    let version_stdout = str::from_utf8(&res.stdout)
         .expect("Cannot convert stdout output of wasm-opt to string")
         .trim();
     let re = Regex::new(r"wasm-opt version (\d+)").expect("invalid regex");
@@ -603,16 +720,15 @@ fn assert_compatible_ink_dependencies(
 ) -> Result<()> {
     for dependency in ["parity-scale-codec", "scale-info"].iter() {
         let args = ["-i", dependency, "--duplicates"];
-        let _ = util::invoke_cargo("tree", &args, manifest_path.directory(), verbosity).map_err(
-            |_| {
+        let _ = util::invoke_cargo("tree", &args, manifest_path.directory(), verbosity, vec![])
+            .map_err(|_| {
                 anyhow::anyhow!(
                     "Mismatching versions of `{}` were found!\n\
                      Please ensure that your contract and your ink! dependencies use a compatible \
                      version of this package.",
                     dependency
                 )
-            },
-        )?;
+            })?;
     }
     Ok(())
 }
@@ -631,7 +747,7 @@ pub fn assert_debug_mode_supported(ink_version: &Version) -> anyhow::Result<()>
     Ok(())
 }
 
-/// Executes build of the smart-contract which produces a wasm binary that is ready for deploying.
+/// Executes build of the smart contract which produces a Wasm binary that is ready for deploying.
 ///
 /// It does so by invoking `cargo build` and then post processing the final binary.
 pub(crate) fn execute(args: ExecuteArgs) -> Result<BuildResult> {
@@ -659,6 +775,14 @@ pub(crate) fn execute(args: ExecuteArgs) -> Result<BuildResult> {
             verbosity,
             " {} {}",
             format!("[1/{}]", build_artifact.steps()).bold(),
+            "Checking ink! linting rules".bright_green().bold()
+        );
+        exec_cargo_dylint(&crate_metadata, verbosity)?;
+
+        maybe_println!(
+            verbosity,
+            " {} {}",
+            format!("[2/{}]", build_artifact.steps()).bold(),
             "Building cargo project".bright_green().bold()
         );
         exec_cargo_for_wasm_target(
@@ -673,7 +797,7 @@ pub(crate) fn execute(args: ExecuteArgs) -> Result<BuildResult> {
         maybe_println!(
             verbosity,
             " {} {}",
-            format!("[2/{}]", build_artifact.steps()).bold(),
+            format!("[3/{}]", build_artifact.steps()).bold(),
             "Post processing wasm file".bright_green().bold()
         );
         post_process_wasm(&crate_metadata)?;
@@ -681,7 +805,7 @@ pub(crate) fn execute(args: ExecuteArgs) -> Result<BuildResult> {
         maybe_println!(
             verbosity,
             " {} {}",
-            format!("[3/{}]", build_artifact.steps()).bold(),
+            format!("[4/{}]", build_artifact.steps()).bold(),
             "Optimizing wasm file".bright_green().bold()
         );
         let optimization_result =
@@ -692,6 +816,20 @@ pub(crate) fn execute(args: ExecuteArgs) -> Result<BuildResult> {
 
     let (opt_result, metadata_result) = match build_artifact {
         BuildArtifacts::CheckOnly => {
+            maybe_println!(
+                verbosity,
+                " {} {}",
+                format!("[1/{}]", build_artifact.steps()).bold(),
+                "Checking ink! linting rules".bright_green().bold()
+            );
+            exec_cargo_dylint(&crate_metadata, verbosity)?;
+
+            maybe_println!(
+                verbosity,
+                " {} {}",
+                format!("[2/{}]", build_artifact.steps()).bold(),
+                "Executing `cargo check`".bright_green().bold()
+            );
             exec_cargo_for_wasm_target(
                 &crate_metadata,
                 "check",
@@ -778,6 +916,20 @@ mod tests_ci_only {
             .any(|e| e.name() == "name")
     }
 
+    /// Creates an executable file at `path` with the content `content`.
+    ///
+    /// Currently works only on `unix`.
+    #[cfg(unix)]
+    fn create_executable(path: &Path, content: &str) {
+        {
+            let mut file = std::fs::File::create(&path).unwrap();
+            file.write_all(content.as_bytes())
+                .expect("writing of executable failed");
+        }
+        std::fs::set_permissions(&path, std::fs::Permissions::from_mode(0o777))
+            .expect("setting permissions failed");
+    }
+
     /// Creates an executable `wasm-opt-mocked` file which outputs
     /// "wasm-opt version `version`".
     ///
@@ -787,14 +939,8 @@ mod tests_ci_only {
     #[cfg(unix)]
     fn mock_wasm_opt_version(tmp_dir: &Path, version: &str) -> PathBuf {
         let path = tmp_dir.join("wasm-opt-mocked");
-        {
-            let mut file = std::fs::File::create(&path).unwrap();
-            let version = format!("#!/bin/sh\necho \"wasm-opt version {}\"", version);
-            file.write_all(version.as_bytes())
-                .expect("writing wasm-opt-mocked failed");
-        }
-        std::fs::set_permissions(&path, std::fs::Permissions::from_mode(0o777))
-            .expect("setting permissions failed");
+        let content = format!("#!/bin/sh\necho \"wasm-opt version {}\"", version);
+        create_executable(&path, &content);
         path
     }
 
@@ -1040,6 +1186,9 @@ mod tests_ci_only {
 
             // then
             assert!(res.is_err());
+
+            // this println is here to debug a spuriously failing CI at the following assert.
+            eprintln!("error: {:?}", res);
             assert!(format!("{:?}", res)
                 .starts_with("Err(Your wasm-opt version is 98, but we require a version >= 99."));
 
@@ -1258,4 +1407,96 @@ mod tests_ci_only {
             Ok(())
         })
     }
+
+    // This test has to be ignored until the next ink! rc.
+    // Before that we don't have the `__ink_dylint_…` markers available
+    // to actually run `dylint`.
+    #[test]
+    #[ignore]
+    fn dylint_must_find_issue() {
+        with_new_contract_project(|manifest_path| {
+            // given
+            let contract = r#"
+            #![cfg_attr(not(feature = "std"), no_std)]
+
+            use ink_lang as ink;
+
+            #[ink::contract]
+            mod fail_mapping_01 {
+                use ink_storage::{traits::SpreadAllocate, Mapping};
+
+                #[ink(storage)]
+                #[derive(SpreadAllocate)]
+                pub struct MyContract {
+                    balances: Mapping<AccountId, Balance>,
+                }
+
+                impl MyContract {
+                    #[ink(constructor)]
+                    pub fn new() -> Self {
+                        Self {
+                            balances: Default::default(),
+                        }
+                    }
+
+                    /// Returns the total token supply.
+                    #[ink(message)]
+                    pub fn get(&self) {
+                        // ...
+                    }
+                }
+            }"#;
+            let project_dir = manifest_path.directory().expect("directory must exist");
+            let lib = project_dir.join("lib.rs");
+            std::fs::write(&lib, contract)?;
+
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                build_artifact: BuildArtifacts::CheckOnly,
+                ..Default::default()
+            };
+
+            // when
+            let res = super::execute(args);
+
+            // then
+            match res {
+                Err(err) => {
+                    eprintln!("err: {:?}", err);
+                    assert!(err.to_string().contains(
+                        "help: add an `initialize_contract` function in this constructor"
+                    ));
+                }
+                _ => panic!("build succeeded, but must fail!"),
+            };
+            Ok(())
+        })
+    }
+
+    #[cfg(unix)]
+    #[test]
+    fn missing_cargo_dylint_installation_must_be_detected() {
+        with_new_contract_project(|manifest_path| {
+            // given
+            let manifest_dir = manifest_path.directory().unwrap();
+
+            // mock existing `dylint-link` binary
+            create_executable(&manifest_dir.join("dylint-link"), "#!/bin/sh\nexit 0");
+
+            // mock a non-existing `cargo dylint` installation.
+            create_executable(&manifest_dir.join("cargo"), "#!/bin/sh\nexit 1");
+
+            // when
+            let args = crate::cmd::build::ExecuteArgs {
+                manifest_path,
+                ..Default::default()
+            };
+            let res = super::execute(args).map(|_| ()).unwrap_err();
+
+            // then
+            assert!(format!("{:?}", res).contains("cargo-dylint was not found!"));
+
+            Ok(())
+        })
+    }
 }
