@@ -1,4 +1,4 @@
-// Copyright 2018-2021 Parity Technologies (UK) Ltd.
+// Copyright 2018-2022 Parity Technologies (UK) Ltd.
 // This file is part of cargo-contract.
 //
 // cargo-contract is free software: you can redistribute it and/or modify
@@ -15,10 +15,6 @@
 // along with cargo-contract.  If not, see <http://www.gnu.org/licenses/>.
 
 pub mod build;
-#[cfg(feature = "extrinsics")]
-mod deploy;
-#[cfg(feature = "extrinsics")]
-mod instantiate;
 pub mod metadata;
 pub mod new;
 pub mod test;
@@ -27,5 +23,6 @@ pub(crate) use self::{
     build::{BuildCommand, CheckCommand},
     test::TestCommand,
 };
-#[cfg(feature = "extrinsics")]
-pub(crate) use self::{deploy::execute_deploy, instantiate::execute_instantiate};
+mod extrinsics;
+
+pub(crate) use self::extrinsics::{CallCommand, InstantiateCommand, UploadCommand};
